`timescale 1ns / 1ps
///////////////////////////////////////////////////////////////////////////////////////////
// Company: Digilent Inc.
// Engineer: Josh Sackos
// 
// Create Date:    07/26/2012
// Module Name:    PmodACL_Demo
// Project Name: 	 PmodACL_Demo
// Target Devices: Nexys3
// Tool versions:  ISE 14.1
// Description: This is a demo for the Digilent PmodACL.  The SW inputs are
//					 used to select either the x-axis, y-axis, or z-axis data for
//					 display, and RST is used to reset the demo.
//
//					 There are four main components in this module, SPIcomponent, sel_Data,
//					 ClkDiv_5Hz, and ssdCtrl.  A START signal is generated by the ClkDiv_5Hz
//					 component, this signal is used to initiate a data transfer between the
//					 PmodACL and the Nexys3.
//
//					 SPIcomponent receives the START signal, configures the PmodACL, and then
//					 receives data pertaining to all three axes.  The data is made available
//					 to the rest of the design on the xAxis, yAxis, and zAxis outputs.
//
//					 These outputs are then sent into the sel_Data component with the the status
//					 of switches SW1 and SW0 on the Nexys3. Depending on the configuration of
//					 these switches, one of the axes data will be selected for display on the
//					 seven segment display. The selected data is converted from 2's compliment
//					 to magnitude, and is then sent to the ssdCtrl where it is converted to a "g"
//					 value and displayed on the SSD with hundredths precision.
//
//					 To select an axis for display configure SW1 and SW0 on the Nexys3 as
//					 shown below.  LED LD0 will illuminate when the x-axis is selected,
//					 LD1 for y-aixs, and LD2 for z-axis.
//
//							SW1	SW0	|	SSD Output	|	LD2	|	LD1	|	LD0
//							------------------------------------------------------
//							off	off	|	x-axis		|	off	|	off	|	on
//							off	on		|	y-axis		|	off	|	on		|	off
//							on		on		|	x-axis		|	off	|	off	|	on
//							on		off	|	z-axis		|	on		|	off	|	off
//
//
//  Inputs:
//		CLK				Onboard system clock
//		RST				Resets the demo
//		SW<0>				Selects y-axis data for display
//		SW<1>				Selects z-axis data for display
//		SDI				Serial Data In
//
//  Outputs:
//		SDO				Serial Data Out
//		SCLK				Serial Clock
//		SS					Slave Select
//		AN					Anodes on SSD
//		SEG				Cathodes on SSD
//		DOT				Cathode for decimal on SSD
//		LED				LEDs on Nexys3
//
// Revision History: 
// 						Revision 0.01 - File Created (Josh Sackos)
///////////////////////////////////////////////////////////////////////////////////////////

// ====================================================================================
// 										  Define Module
// ====================================================================================
module PmodACL_Demo(
		CLK,
		RST,
		SW,
		SDI,
		SDO,
		SCLK,
		SS,
		AN,
		SEG,
		DOT,
		LED
);

// ====================================================================================
// 										Port Declarations
// ====================================================================================
   input        CLK;
   input        RST;
   input [1:0]  SW;
   input        SDI;
   output       SDO;
   output       SCLK;
   output       SS;
   output [3:0] AN;
   output [6:0] SEG;
   output       DOT;
   output [2:0] LED;
   
// ====================================================================================
// 								Parameters, Register, and Wires
// ====================================================================================
   
   wire [9:0]   xAxis;		// x-axis data from PmodACL
   wire [9:0]   yAxis;		// y-axis data from PmodACL
   wire [9:0]   zAxis;		// z-axis data from PmodACL
   
   wire [9:0]   selData;	// Data selected to display
   
   wire         START;		// Data Transfer Request Signal
   
//  ===================================================================================
// 							  				Implementation
//  ===================================================================================
   
		//-----------------------------------------------
		//	Select Display Data and Convert to Magnitude
		//-----------------------------------------------
		sel_Data SDATA(
					.CLK(CLK),
					.RST(RST),
					.SW(SW),
					.xAxis(xAxis),
					.yAxis(yAxis),
					.zAxis(zAxis),
					.DOUT(selData),
					.LED(LED)
		);
		
		//-----------------------------------------------
		//		 			 Interfaces PmodACL
		//-----------------------------------------------
		SPIcomponent SPI(
					.CLK(CLK),
					.RST(RST),
					.START(START),
					.SDI(SDI),
					.SDO(SDO),
					.SCLK(SCLK),
					.SS(SS),
					.xAxis(xAxis),
					.yAxis(yAxis),
					.zAxis(zAxis)
		);
		
		//-----------------------------------------------
		//		 	 Formats Data and Displays on SSD
		//-----------------------------------------------
		ssdCtrl Disp(
					.CLK(CLK),
					.RST(RST),
					.DIN(selData[9:0]),
					.AN(AN),
					.SEG(SEG),
					.DOT(DOT)
		);
		
		//-----------------------------------------------
		//	 Generates a 5Hz Data Transfer Request Signal
		//-----------------------------------------------
		ClkDiv_5Hz genStart(
					.CLK(CLK),
					.RST(RST),
					.CLKOUT(START)
		);
   
endmodule
